      DOUBLE PRECISION FUNCTION DAQUAD(RULE,N,F,A,B,TOL,MAXLEV,SING,
     *                                 ERREST)

C   FOR COMPLETE DOCUMENTATION SEE AQUAD.
C
      DOUBLE PRECISION QRIGHT(30),XSAVE(30),F,DQUAD,
     *                 A,B,TOL,SING,ERREST,FAC,SUM,CORR,FLAG,
     *                 XL,XR,TOLPU,XM,STEP, TEMP
      INTEGER LSAVE(30)
      EXTERNAL RULE,F

	double precision l_qright, l_qleft, l_qnow, l_qprev, l_qdiff,
     +			 r_qright, r_qleft, r_qnow, r_qprev, r_qdiff,
     +			 totalerr, err, totalval, val

	double precision  h_left(1000), h_right(1000),
     +			  h_value(1000), h_error(1000), 
     +			  h_lvalue(1000), h_rvalue(1000)


	integer i, heap(0:1000), pos
C
C      ***************
C      *
C      * STRAIGHT INTEGRATION IF MAXLEV = 0.
C      *
C      ***************
C
         IF (MAXLEV .NE. 0) GO TO 4
              DAQUAD = DQUAD(RULE,N,F,A,B)
      RETURN
C
C      ***************
C      *
C      * OBTAIN  THE  APPROPRIATE SCALING FACTOR  FAC  (FOR THE
C      * CONVERGENCE CRITERION AND THE ERROR ESTIMATE) FROM THE
C      * SUBROUTINE DEFINED BY RULE.
C      *
C      ***************
C
   4     NF = -N
         CALL RULE(NF,A,B,XSAVE,QRIGHT)
         FAC = XSAVE(1)

C
C      ***************
C      *
C      * INITIALIZATION AND CALCULATIONS FOR THE ZERO-TH LEVEL.
C      *
C      ***************
C
         SUM = 0.0D0
         CORR = 0.0D0
         ERREST = 0.0D0
	 err=0.0d0
	 totalerr = 0.0d0
         FLAG = 0.0D0
         SING = 0.0D0
         ICOUNT = 0
         XL = A
         XR = B
         tolpu = tol*fac
         l_qprev= DQUAD(RULE,N,F,A,B)
	 heap(0)=0

C
C      ***************
C      *
C      * SUCCESSIVELY BISECT THE SUBINTERVALS UNTIL EITHER THE MAXIMUM
C      * ALLOWED LEVEL IS REACHED OR THE INTEGRAL ON A SUBINTERVAL
C      * AGREES WITH THE SUM OF THE INTEGRALS ON ITS 2 HALVES.
C      *
C      ***************
C

	xm = 0.5d0*(xl + xr)
	call calc_vals(l_qprev, xl, xr, rule, n, f,
     +                 l_qleft, l_qright, l_qnow, l_qdiff)
	 totalerr = l_qdiff
	 totalval=l_qnow
	 val=l_qnow
C
C      ***************
C      *
C      * TEST FOR INTERVAL CONVERGENCE AND CHECK IF THE MAXIMUM LEVEL
C      * IS REACHED.
C      *
C      ***************
C
	if (dabs(totalerr) .le. tolpu) then
	  go to 30
	endif
            l_qprev = l_qleft
	    r_qprev = l_qright
	    pos = 1
	    err = l_qdiff

	heap(0)=1

10	call calc_vals (l_qprev, xl, xm, rule, n, f,
     +		  l_qleft, l_qright, l_qnow, l_qdiff)
	call calc_vals (r_qprev, xm, xr, rule, n, f,
     +		  r_qleft, r_qright, r_qnow, r_qdiff)

	  if (heap(0) .ge. 999) go to 20
	    if (dabs(l_qdiff) .gt. dabs(r_qdiff)) then
	      call insert_top(xl, xm, l_qnow, l_qdiff, l_qleft, l_qright,
     +			 heap, h_left, h_right, h_value, h_error, 
     +			 h_lvalue, h_rvalue, pos)
	      call insert(xm, xr, r_qnow, r_qdiff, r_qleft, r_qright,
     +			 heap, h_left, h_right, h_value, h_error, 
     +			 h_lvalue, h_rvalue)

	    else
	      call insert_top(xm, xr, r_qnow, r_qdiff, r_qleft, r_qright,
     +			 heap, h_left, h_right, h_value, h_error, 
     +			 h_lvalue, h_rvalue, pos)
	      call insert(xl, xm, l_qnow, l_qdiff, l_qleft, l_qright,
     +			 heap, h_left, h_right, h_value, h_error, 
     +			 h_lvalue, h_rvalue)
	    endif


	    totalerr=totalerr - err + l_qdiff +r_qdiff
	    totalval=totalval - val + l_qnow +r_qnow
            if (dabs(totalerr) .le. tolpu) go to 30

	call remove_top(xl, xr, val, err, l_qprev, r_qprev, heap,
     +		   h_left, h_right, h_value, h_error, h_lvalue, h_rvalue)


	xm=0.5d0*(xl+xr)
	go to 10

C
C      ***************
C      *
C      * MAXIMUM LEVEL WAS REACHED. ACCUMULATE THE TOTAL LENGTH OF THE
C      * SUBINTERVALS ON WHICH THE CONVERGENCE FAILED, AND RECORD THE
C      * LOCATION OF THE SINGULARITY.
C      *
C      ***************
C

20	print *, "out of memory", heap(0)

C
C      ***************
C      *
C      * THE INTEGRAL ON THE INTERVAL CONVERGED OR THE MAXIMUM ALLOWED
C      * LEVEL WAS REACHED.  ADD CONTRIBUTION INTO THE RUNNING SUMS.
C      *
C      ***************
C

c30	sum=totalval
c	corr=totalerr
30
c	print *, "totalval=", totalval
c	do 35 i=1, heap(0), 1
c    	  sum = sum + h_value(i)
c	  print *, h_value(i)
c          corr = corr + h_error(i)
c35	continue

c	print *, "sum=", sum

C
C      ***************
C      *
C      * ADD THE CORRECTION TERM FROM THE FIRST STEP OF RICHARDSON
C      * EXTRAPOLATION.  MAKE SURE ERREST IS NOT LESS THAN ROUNDOFF
C      * LEVEL.
C      *
C      ***************
C

40	corr=totalerr

         DAQUAD = totalval + CORR/FAC
         ERREST = totalerr/FAC
         TOL = FLAG
         MAXLEV = ICOUNT*2
         IF (ERREST .EQ. 0.0D0) RETURN
  50     TEMP = DABS(DAQUAD) + ERREST
         IF (TEMP .NE. DABS(DAQUAD)) RETURN
         ERREST = 2.0D0*ERREST
         GO TO 50


      END
